/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005-2006 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>

#include "sambascanner.h"
#include "Tools.h"
#include "Processes.h"
#include "String.h"
#include "Parser.h"
#include "Mutex.h"
#include "Threads.h"

void* Redirect_Thread(void *Parameters) {
	struct Redirect_Thread_InputData *Input = (struct Redirect_Thread_InputData*)Parameters;
	char Buffer[BUFSIZE_SHORT];
	int Read;
	int fd;
	
	while (!Input->Active) usleep(100 * 1000);
	
	fd = open(Input->Outfile, O_CREAT | O_TRUNC | O_WRONLY, 0644);
	if (fd == -1) {
		message(1, "Cannot open redirector output file '%s' for writing: %s\n", Input->Outfile, strerror(errno));
		return NULL;
	}

	do {
		Read = read(Input->Read_FD, Buffer, sizeof(Buffer));
		if (Read > 0) {
			double Time;
			write(fd, Buffer, Read);
			Time = GetTime();
			(*Input->Processes)[Input->ProcessID].TermTime = Time + (Input->TermTime * Options.Timings);
			(*Input->Processes)[Input->ProcessID].KillTime = Time + ((Input->TermTime + Input->KillTime) * Options.Timings);
		}
	} while (Read > 0);

	free(Input);
	pthread_exit(NULL);
	return NULL;
}

void* Process_Watcher_Thread(void *Parameters) {
	struct Process_Watcher_Thread_InputData *Input = (struct Process_Watcher_Thread_InputData*)Parameters;
	int i;
	double Time;
	
	while (Input->Running) {
		Time = GetTime();
		for (i = 0; i < *(Input->NumberProcesses); i++) {
			if (((*Input->Processes)[i].Running == 1) && ((*Input->Processes)[i].Valid == 1)) {
				if (Time > (*Input->Processes)[i].KillTime) {
					message(3, "=> SIGKILL Processes[%d] [PID %d]\n", i, (*Input->Processes)[i].PID);
					(*Input->Processes)[i].Signal = SIGKILL;
					kill((*Input->Processes)[i].PID, SIGKILL);
				} else {
					if (Time > (*Input->Processes)[i].TermTime) {
						message(3, "=> SIGTERM Processes[%d] [PID %d]\n", i, (*Input->Processes)[i].PID);
						(*Input->Processes)[i].Signal = SIGTERM;
						kill((*Input->Processes)[i].PID, SIGTERM);
					}
				}
			}
		}
		usleep(100*1000);	/* 100ms */
	}
	Input->Running = -1;
	pthread_exit(NULL);
	return NULL;
}

void* Process_Host_Thread(void *Parameters) {
	struct Process_Host_Thread_InputData *Input = (struct Process_Host_Thread_InputData*)Parameters;
	char Buffer[BUFSIZE_SHORT];
	char FileBuffer[BUFSIZE_SHORT];
	int Result, ProcID;

	strcpy(FileBuffer, TEMPPATH "sambascanner_process_hostXXXXXX");
	mkstemp(FileBuffer);

	message(2, "Processing host: '%s'\n", Input->Hostname);
	snprintf(Buffer, sizeof(Buffer), "-g -N -L //%s", Input->Hostname);
	ProcID = Exec_Timeout(Options.SmbClient_Executable, Buffer, 5.0, 2.0, FileBuffer, -1, Input->Processes, Input->NumberProcesses);
	Result = WaitFor(ProcID, Input->Processes);

	pthread_mutex_lock(Input->NumberMutex);
	if (Result < 0) {
		/* Process was killed */
		*(Input->Hostlookup_Processes_Failed)++;
	} else {
		*(Input->Hostlookup_Processes_Succeeded)++;
	}
	pthread_mutex_unlock(Input->NumberMutex);

	Parse_Hostinfo(Input->Hostname, FileBuffer, Input->Hostlist, Input->Sharelist);
	unlink(FileBuffer);
	ThreadCnt_Dec();

	pthread_exit(NULL);
	return NULL;
}

void* Process_Share_Thread(void *Parameters) {
	struct Process_Share_Thread_InputData *Input = (struct Process_Share_Thread_InputData*)Parameters;
	char Buffer[BUFSIZE_SHORT];
	char FileBuffer[BUFSIZE_SHORT];
	int Result, ProcID;

	strcpy(FileBuffer, TEMPPATH "sambascanner_process_sharesXXXXXX");
	mkstemp(FileBuffer);
	Add_String_Atomic(Input->Resultfilelist, FileBuffer);
	
	message(2, "Now revealing content of smb:%s with unique thread ID %d\n", Input->Sharename, Input->UniqueThreadID);
	snprintf(Buffer, sizeof(Buffer), "smb:%s", Input->Sharename);
	ProcID = Exec_Timeout(Options.SmbLister_Executable, Buffer, 90.0, 5.0, FileBuffer, Input->UniqueThreadID, Input->Processes, Input->NumberProcesses);
	Result = WaitFor(ProcID, Input->Processes);
	
	ThreadCnt_Dec();
	pthread_exit(NULL);
	return NULL;
}

