/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005-2006 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include <stdio.h>
#include <errno.h>
#include <sys/time.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <libsmbclient.h>

#include "String.h"

#define DEBUG_LEVEL 	99
#define BUFSIZE			1024

static void Authentication_Function(const char *pServer, const char *pShare, char *pWorkgroup, int workgroup_len, char *pUsername, int username_len, char *pPassword, int password_len) {
    strncpy(pUsername, "anonymous", username_len);
    strncpy(pPassword, "password", password_len);
}

int main(int argc, char **argv) {
	int dir;
	struct smbc_dirent *Directory_Entry;
	char *Url;
	int i;
	char Buffer[BUFSIZE];
	StringList Dirlist;

	if (argc != 2) {
		fprintf(stderr, "Syntax: %s [Sharename]\n", argv[0]);
		fprintf(stderr, "Example: %s smb://NEO/PICTURES\n", argv[0]);		
		return 1;
	}

	if (smbc_init(Authentication_Function, DEBUG_LEVEL) != 0) {
		fprintf(stderr, "Could not initialize smbc-library.\n");
		return 2;
	}

	Init_StringList(&Dirlist);
	Add_String(&Dirlist, argv[1]);
	for (i = 0; i < Dirlist.Entries; i++) {
		Url = Dirlist.Entry[i];
		fprintf(stderr, "Opening (%s)...\n", Url);
		if ((dir = smbc_opendir(Url)) < 0) {
			fprintf(stderr, "Could not open [%s] (%d:%s)\n", Url, errno, strerror(errno));
			continue;
		}
		fprintf(stderr, "Connection opened with smbc_opendir().\n");

		while ((Directory_Entry = smbc_readdir(dir)) != NULL) {
			if (Directory_Entry->smbc_type==SMBC_DIR) {
				if ((strcmp(Directory_Entry->name, ".")!=0) && (strcmp(Directory_Entry->name, "..")!=0)) {
					snprintf(Buffer, BUFSIZE, "%s/%s", Url, Directory_Entry->name);
					Add_String_Unique(&Dirlist, Buffer);
				}
			} else 	if (Directory_Entry->smbc_type == SMBC_FILE) {
				printf("%s/%s\n", Url, Directory_Entry->name);
			}
		}
		smbc_closedir(dir);
	}

	return 0;
}

